package com.drinkertea.test2smack;

import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.os.Build;
import android.os.IBinder;

import androidx.core.app.NotificationCompat;
import androidx.core.app.TaskStackBuilder;

import org.jivesoftware.smack.MessageListener;
import org.jivesoftware.smack.chat2.Chat;
import org.jivesoftware.smack.chat2.IncomingChatMessageListener;
import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smackx.muc.MultiUserChat;
import org.jxmpp.jid.EntityBareJid;
import org.jxmpp.jid.impl.JidCreate;

import java.util.Locale;


public class NotificationService extends Service {

    public static boolean serverIsWorked = false;
    NotificationCompat.Builder builder;
    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        try {
            if(!serverIsWorked){
                new Thread(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            task();
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                }).start();
                serverIsWorked = true;
                builder = new NotificationCompat.Builder(this, "My channel")
                        .setContentText("Notification server is running!")
                        .setContentTitle("Assist chat")
                        .setSmallIcon(R.mipmap.ic_launcher_round);
                startForeground(1001, builder.build());
            }
        } catch (Exception e) {
            //ErrorLogFile.WriterLogs(e.getMessage());
            System.out.println(e.getMessage());
        }
        return super.onStartCommand(intent, flags, startId);
    }

    @Override
    public void onCreate() {
        super.onCreate();
        try {
            if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.O){
                NotificationChannel channel = new NotificationChannel("My channel", "My notification", NotificationManager.IMPORTANCE_DEFAULT);
                NotificationManager manager = getSystemService(NotificationManager.class);
                manager.createNotificationChannel(channel);
            }
            System.out.println("Server started!");
        } catch (Exception e) {
            ErrorLogFile.WriterLogs(e.getMessage());
            //System.out.println(e.getMessage());
        }
    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    private void task(){
        try {
            XMPPClass.chatManager.addIncomingListener(new IncomingChatMessageListener() {
                @Override
                public void newIncomingMessage(EntityBareJid from, Message message, Chat chat) {
                    System.out.println("New message from:" + from + ": " + message.getBody());
                    try {
                        createNotificationChannel(from.asBareJid().toString(), message.getBody());
                    }catch (Exception ex){
                        //ErrorLogFile.WriterLogs(ex.getMessage());
                        System.out.println(ex.getMessage());
                    }
                }
            });
            for (MultiUserChat muc : XMPPClass.listForMessageListenerMuc){
                muc.addMessageListener(new MessageListener() {
                    @Override
                    public void processMessage(Message message) {
                        try {
                            if (message.getBody() != null){
                                createNotificationChannel(message.getFrom().toString().split("@")[0].toUpperCase(Locale.ROOT) + "/" + message.getFrom().toString().split("/")[1], message.getBody());
                            }

                        }catch (Exception ex){
                            //ErrorLogFile.WriterLogs(ex.getMessage());
                            System.out.println(ex.getMessage());
                        }
                    }
                });
            }
        } catch (Exception e) {
            ErrorLogFile.WriterLogs(e.getMessage());
            //System.out.println(e.getMessage());
        }
    }


    private void createNotificationChannel(String from, String message) {
        try {
            XMPPClass.chatJid = JidCreate.entityBareFrom(from);
            Intent resultIntent = new Intent(this, ChatsRooms.class);
            TaskStackBuilder stackBuilder = TaskStackBuilder.create(this);
            stackBuilder.addNextIntentWithParentStack(resultIntent);
            PendingIntent resultPendingIntent =
                    stackBuilder.getPendingIntent(0,
                            PendingIntent.FLAG_UPDATE_CURRENT | PendingIntent.FLAG_IMMUTABLE);

            NotificationCompat.Builder builder = new NotificationCompat.Builder(NotificationService.this, "My channel");
            builder.setContentTitle(from);
            builder.setContentText(message);
            builder.setSmallIcon(R.mipmap.ic_launcher);
            builder.setAutoCancel(true);
            builder.setContentIntent(resultPendingIntent);

            startForeground(1001, builder.build());
        } catch (Exception e) {
            ErrorLogFile.WriterLogs(e.getMessage());
            //System.out.println(e.getMessage());
        }

    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        try {
            serverIsWorked = false;
            System.out.println("Server destroy!");
        } catch (Exception e) {
            ErrorLogFile.WriterLogs(e.getMessage());
            //System.out.println(e.getMessage());
        }

    }
}
